/*
 * Cliente ORB en Java para curso de Delphi y de Java
 * 
 */
import ORBDelCurso.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

import org.omg.CORBA.*;

public class ClienteORB extends Frame {

    Button btnConectar;
    Button btnQuienEres;
    Label lblDeCabeza;
    TextField   edtDeCabeza;
    Button btnDeCabeza;
    TextArea mmLog;
    Label estatus;
    ORB orb;
    boolean usarBind = false;

    /* Esta es la conexion al objeto en el servidor */
    ObjetoCursoFactory fac;
    IObjetoCurso obj;

    /** 
     * Inicializa la clase y aade la interfaz de usuario.
     */
    public ClienteORB() {
	
	fac = null;

	/* Inicializamos la interface con todos los botones y cosillas */
	btnConectar = new Button("Conectar");
	btnQuienEres = new Button("Tu Quien Eres?");
	btnDeCabeza = new Button("De Cabeza!");
	lblDeCabeza = new Label("Cadena que pondremos de cabeza:");
        edtDeCabeza = new TextField();
	estatus = new Label("Desconectado.");

	mmLog = new TextArea();

	Panel pnl = new Panel();
	pnl.setLayout( new BorderLayout() );
	
	Panel toppnl = new Panel();
	toppnl.setLayout( new BorderLayout() );
	pnl.add(toppnl, BorderLayout.NORTH);
	toppnl.add(lblDeCabeza, BorderLayout.WEST);
	toppnl.add(edtDeCabeza, BorderLayout.CENTER);
	toppnl.add(btnDeCabeza, BorderLayout.EAST);
	pnl.add(mmLog, BorderLayout.CENTER);

	Panel pnlNorte = new Panel();
	pnlNorte.add(btnConectar);
	pnlNorte.add(btnQuienEres);

	setLayout(new BorderLayout());

	add(pnlNorte, BorderLayout.NORTH);
	add(pnl, BorderLayout.CENTER);
	
	add(estatus, BorderLayout.SOUTH);
	Actualizar();

	ActionListener list = new ClienteListener();
	btnConectar.addActionListener(list);
	btnDeCabeza.addActionListener(list);
	btnQuienEres.addActionListener(list);

	setTitle("Cliente en Java ORB - Curso Hackerdude.com");
	pack();
	show();

    };

    /**
     * Esta funcin obtiene una conexin, ya sea utilizando 
     * Bind (con visibroker) o utilizando un archivo de textio .ior con
     * el puntero de interfase directamente. Una vez que tiene 
     * la fbrica de clases, ejecuta CreateInstance para obtener
     * un IObjetoCurso en nuestro puntero.
    */
    public void ObtenConexion(boolean ConBind) { 
	
	if ( ConBind ) {
	    ObtenConexionConBind();
	} else { 
	    ObtenConexionConArchivo();
	};
	if ( fac!=null ) {
	    obj = fac.CreateInstance("");
	};
	Actualizar();    

    };

    /**
     * Esta es la manera estndar de hacer una conexin con 
     * el servicio de nombres de Visibroker.
     */
    private void ObtenConexionConBind() { 
	fac = ObjetoCursoFactoryHelper.bind(orb, null);
    };

    /**
     * Este cdigo crea una instancia de ObjetoCurso leyendo el
     * puntero al objeto de un archivo que el servidor debi haber
     * escrito con anterioridad. Es til si no tienes Visibroker pero
     * quieres experimentar de todas maneras. Obviamente si tu
     * servidor no est utilizando visibroker, otras aplicaciones
     * que usan visibroker para encontrar tus objetos tendrn que
     * leer el archivo IOR que has generado en el servidor para poder
     * comunicarse.
     **/
    private void ObtenConexionConArchivo() {
	System.out.println("Abriendo ObjetoCursoFactory.ior...");
	String ior = null;
	try
	    {
		BufferedReader br = new BufferedReader( new FileReader( "../ObjetoCursoFactory.ior") );
		ior = br.readLine();
		br.close();
		
	    } catch ( IOException ie ) {
		System.err.println("No pude leer el IOR del ObjetoCurso...!\n\n");
		ie.printStackTrace(System.err);
       };

       // Ahora que lo hemos leido, necesitamos convertirlo a un ObjetoCurso
	System.out.println(ior);
       org.omg.CORBA.Object object = orb.string_to_object( ior );
       fac = ObjetoCursoFactoryHelper.narrow( object );
       

    };

    /**
     * Actualiza la pantalla. Si estamos desconectados hace que 
     * desactiva los botones.
     */
    public void Actualizar() {

	if ( fac == null ) { estatus.setText("Desconectado."); 
	} else { estatus.setText("Conectado!"); }

	btnQuienEres.setEnabled(fac!=null);
	edtDeCabeza.setEnabled(fac!=null);
	btnDeCabeza.setEnabled(fac!=null);
	mmLog.setEnabled(fac!=null);

    };

    /** Esto es P'a que corra :-) El main, pues. Crea el objeto,
     * inicializa el ORB con los parmetros y decide si vamos a usar 
     * bind o no,
     */
    public static void main( String args[] ) {
	
	ClienteORB cliente = new ClienteORB();
	cliente.addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) { System.exit(0); }
	});
	
	cliente.orb = ORB.init(args, null);
	System.out.println("ORB Inicializado");
	
	if ( args.length > 0 ) {

	    if (args[0].equals("usabind")) { cliente.usarBind = true; };
	    if (args[0].equals("usaior")) { cliente.usarBind = false; };
	}; 
	
	if ( cliente.usarBind ) 
	    { System.out.println("Usando Bind..."); } 
	else { System.out.println("Usando Archivo IOR..."); };
	
    };

    /**
     * Esta clase responde a los botones.
     */
    public class ClienteListener implements ActionListener {

	public void actionPerformed(ActionEvent ev) {
	    
	    String command = ev.getActionCommand();
	    
	    if ( command.equals("Conectar")) {
		/* Al conectar, checamos si ya 
		   estamos conectados. Si no, obtenemos una 
		   conexin. De otro modo desconectamos 
		   (limpiando nuestros punteros) y actualizamos 
		   la pantalla. */
		if ( fac == null ) {
		    ObtenConexion(usarBind);
		} else {
		    obj = null;
		    fac = null;
		    Actualizar();
		};
	    }
	    /* Ejecutamos "TuQuienEres" en el objeto remoto */
	    if ( command.equals("Tu Quien Eres?") ) {
		mmLog.append(obj.TuQuienEres()+'\n');
	    };
	    /* Aadimos al TextArea la cadena de cabeza, 
	       de acuerdo al objeto remoto. */
	    if ( command.equals("De Cabeza!") ) {
		System.out.println(edtDeCabeza.getText());
		mmLog.append(obj.DeCabeza(edtDeCabeza.getText())+'\n');
	    };
	};


    };

};




